%% Find loop transfer function: L = PC
% System Open-Loop Parameters
K = 23.2; % DC motor steady-state gain (V-s/rad)
tau = 0.1; % DC motor time constant (s)
% 
w = logspace(0,3,1000); % frequency range (rad/s)
s = 1i * w; % Laplace operator: s = j*w
h = 0.01; % sampling interval (s)
% 
% PI Design Parameters
wn = 15; 
zeta = 0.8;
% PI controller
kp = ( 2 * zeta * wn * tau - 1 ) / K; 
ki = wn^2 * tau / K;    
% 
% Transfer function
wf = 100; % filter cutoff frequency (rad/s)
Filter = wf./(s+wf);
P = K./(tau*s+1).*exp(-h*s); 
P=P.*Filter; % plant with filter and sampling effects
C = kp+ki./s; % PI compensator
L = P .* C; % loop transfer function
% 
%% Nyquist plot
% Nyquist data
RealL = real(L);
ImagL = imag(L);
MagL = abs(L);
% 
figure(1);
plot( RealL, ImagL, '-k' );
hold on;
% plot( RealL(i_PM), ImagL(i_PM), 'ko' ); 
% plot( RealL(i_GM), ImagL(i_GM), 'ko');   
% 
% cross
plot( [-1.1 1.1], [0 0], 'k' );
plot([0 0 ],[-1.1 1.1], 'k' );
% 
% circle
theta = 0:.1:2.1*pi;
xc = sin(theta);
yc = cos(theta);
plot(xc,yc,'--k');
% 
axis([-1.1 1.1 -1.1 1.1]);
xlabel('real axis');
ylabel('j\omega axis');
axis square;
hold off;
