%
% clear workspace
clear;
% 
% QUBE-Servo System Open-Loop Parameters
K = 22.4;      % [V.s/rad]
tau = 0.15;    % [s]
% 
w = logspace(0,3,1000);
s = 1i * w;
h = 0.01;
% w_ol= 1 / tau;
% 
% Transfer function
wf = 50;
Filter = wf ./ ( s + wf );
% 
% select effects to add
% design_type = 1; % ideal
% design_type = 2; % with sampling
design_type = 3; % with sampling and filtering
% 
iMs = 1;
% find maximum sensitivity value for a range of natural frequencies, wn
for wn = 2.5:0.5:20
    % damping ratio
    zeta= .8;
    % find gains
    kp = ( 2 * zeta * wn * tau - 1 ) / K; % [V.s/rad]
    ki = wn^2 * tau / K;    % [V/rad]
    %
    % Ideal Plant
    if (design_type == 1) 
        P = K ./ ( tau * s + 1 );
    end
    % Plant with sampling effect
    if (design_type == 2) 
        P = K ./ ( tau * s + 1 ) .* exp(-h*s);        
    end
    % Plant with sampling and filtering effect
    if (design_type == 3) 
        P = K ./ ( tau * s + 1 ) .* exp(-h*s); P = P.*Filter; 
    end
    %     
    % PI Compensator
    C = kp + ki ./ s;
    % Sensitivity function
    S = 1 ./ ( 1 + P .* C );
    % Find max sensitivy, Ms, and the index for it
    [xx,ixx] = max(abs(S));
    % Record 'wn' associated with max Ms value above
    wnSMax(iMs) = wn; 
    % maximum sensitiviy value for that wn
    Smax(iMs) = xx;
    iMs= iMs + 1;
end
% 
% plot max sensitivy value over closed-loop frequencies
plot( wnSMax, Smax, 'Color', 'k', 'LineStyle', '-' );
xlabel( '\omega_n (rad/s)' );
ylabel( 'M_s' );
% 
% Find minimum sensitivy value and corresponding 'wn' value
[xx,ixx] = min(Smax);
wn_Smin = wnSMax(ixx);
% display results
fprintf(' \nThe best design is at wn = %5.2f resulting in Max(S) = %5.2f\n',wn_Smin,xx)
% resulting PI gains for MSD
kp = ( 2 * zeta * wn_Smin * tau - 1 ) / K; % [V.s/rad]
ki = wn_Smin^2 * tau / K;    % [V/rad]
fprintf(' Resulting gains: kp = %5.3g and ki = %5.3g\n',kp,ki)
    
    


	
	